/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel21-tictactoe-core.
 *
 * ipspiel21-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel21-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel21-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain;

import de.fhdw.gaming.core.domain.GameBuilder;
import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.core.domain.ObserverFactoryProvider;

/**
 * A builder which allows to create a Vier Gewinnt game.
 */
public interface VierGewinntGameBuilder extends GameBuilder {

    /**
     * The default number of columns.
     */
    int DEFAULT_COLUMN_SIZE = 7;

    /**
     * The default number of rows.
     */
    int DEFAULT_ROW_SIZE = 6;

    /**
     * Creates an {@link VierGewinntPlayerBuilder} which allows to create and add a
     * player to the game together with her strategy.
     */
    VierGewinntPlayerBuilder createPlayerBuilder();

    /**
     * Adds a player builder and the corresponding strategy.
     *
     * @param playerBuilder The builder player used for creating the player.
     * @param strategy      The player's strategy.
     * @throws GameException if adding the player is not allowed by the rules of the
     *                       game.
     */
    VierGewinntGameBuilder addPlayerBuilder(VierGewinntPlayerBuilder playerBuilder, VierGewinntStrategy strategy)
            throws GameException;

    /** 
     *  Changes the columnSize to the parameter.
     * @param columnSize
     * @return
     */
    VierGewinntGameBuilder changeColumnSize(int columnSize);

    /** 
     * Changes the rowSize to the parameter.
     * @param rowSize 
     * @return
     */
    VierGewinntGameBuilder changeRowSize(int rowSize);

    /**
     * Changes the {@link ObserverFactoryProvider}.
     *
     * @param newObserverFactoryProvider The new {@link ObserverFactoryProvider}.
     * @return {@code this}
     */
    VierGewinntGameBuilder changeObserverFactoryProvider(ObserverFactoryProvider newObserverFactoryProvider);

    @Override
    VierGewinntGame build(int id) throws GameException, InterruptedException;
}
