/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel21-tictactoe-core.
 *
 * ipspiel21-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel21-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel21-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain;

import de.fhdw.gaming.core.domain.GameException;

/**
 * A builder which allows to create a VierGewinnt player.
 */
public interface VierGewinntPlayerBuilder {

    /**
     * Changes the name of the player.
     * <p>
     * There is no default.
     *
     * @param newName The name of the player.
     * @return {@code this}
     */
    VierGewinntPlayerBuilder changeName(String newName);

    /**
     * Changes if the next build player will be yellow as color or not.
     * 
     * @param yellow
     * @return
     */
    VierGewinntPlayerBuilder changeUsingYellow(boolean yellow);

    /**
     * Will the next build player be using yellow?
     * @return true if the playerBuilder currently will build yellow-using players.
     */
    boolean isUsingYellow();

    /**
     * Builds the player.
     *
     * @param state The VierGewinnt game state.
     * @return The VierGewinnt player.
     * @throws GameException if creating the player is not allowed by the rules of
     *                       the game.
     */
    VierGewinntPlayer build(VierGewinntState state) throws GameException;
}
