/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel21-tictactoe-core.
 *
 * ipspiel21-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel21-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel21-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain;

import de.fhdw.gaming.core.domain.State;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.moves.VierGewinntMove;

/**
 * Represents the state of a VierGewinnt game.
 */
public interface VierGewinntState extends State<VierGewinntPlayer, VierGewinntState> {

    /**
     * Returns the board.
     */
    VierGewinntBoard getBoard();

    /**
     * Returns the player using yellow.
     */
    VierGewinntPlayer getYellowPlayer();

    /**
     * Returns the player using red.
     */
    VierGewinntPlayer getRedPlayer();

    /**
     * Returns the currently active player, i.e. the player that needs to make the next move.
     */
    VierGewinntPlayer getCurrentPlayer();

    /**
     * Indicates that a move has been completed.
     * @param move
     */
    void moveCompleted(VierGewinntMove move);
}
