package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.impl;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntBoard;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntDirection;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntField;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntFieldState;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntPosition;

/**
 * The Implementation of a VierGewinnt Field.
 *
 * @author Robby Rabbitman
 *
 */
public class VierGewinntFieldImpl implements VierGewinntField {

    /**
     * State of this field.
     *
     */
    private VierGewinntFieldState state;
    /**
     * Position of this field.
     */
    private final VierGewinntPosition position;
    /**
     * Board of this Field.
     */
    private final VierGewinntBoard board;

    /**
     * Empty field.
     *
     * @param position
     * @param board
     * @throws GameException
     */
    public VierGewinntFieldImpl(final VierGewinntPosition position, final VierGewinntBoard board)
            throws GameException {
        super();
        if (board.getFields().stream().map(VierGewinntField::getPosition).filter(p -> p.equals(position)).findAny()
                .isPresent()) {
            throw new GameException("Position already used!");
        }
        this.board = board;
        this.position = position;
        this.state = VierGewinntFieldState.EMPTY;
    }

    /**
     * Copy of source.
     *
     * @param source The field to copy.
     * @param board  The board to associate with this field.
     */
    public VierGewinntFieldImpl(final VierGewinntFieldImpl source, final VierGewinntBoard board) {
        this.board = board;
        this.position = source.getPosition();
        this.state = source.getState();
    }

    @Override
    public VierGewinntBoard getBoard() {
        return this.board;
    }

    @Override
    public VierGewinntPosition getPosition() {
        return this.position;
    }

    @Override
    public VierGewinntFieldState getState() {
        return this.state;
    }

    @Override
    public boolean hasNeighbour(final VierGewinntDirection direction) {
        try {
            this.getBoard().getFieldAt(direction.step(this.getPosition()));
            return true;
        } catch (final IllegalArgumentException e) {
            return false;
        }

    }

    @Override
    public VierGewinntField getNeighbour(final VierGewinntDirection direction) throws IllegalArgumentException {
        return this.getBoard().getFieldAt(direction.step(this.getPosition()));
    }

    @Override
    public void changeState(final VierGewinntFieldState newState) throws GameException {
        this.state = newState;
    }

    @Override
    public VierGewinntField deepCopy(final VierGewinntBoard target) {
        return new VierGewinntFieldImpl(this, target);
    }

    @Override
    public String toString() {
        return "[state=" + this.state + ", position=" + this.position + "]";
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj == null || obj.getClass() != this.getClass()) {
            return false;
        } else if (obj == this) {
            return true;
        }

        final VierGewinntFieldImpl otherField = (VierGewinntFieldImpl) obj;

        if (otherField.getBoard().getRowSize() != this.getBoard().getRowSize()
                || otherField.getBoard().getColumnSize() != this.getBoard().getColumnSize()) {
            return false;
        }

        return otherField.getPosition().equals(this.getPosition()) && otherField.getState() == this.state;

    }

    /**
     * Hashcode of this.
     */
    @Override
    public int hashCode() {
        return this.position.hashCode();
    }
}
