package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.impl;

import java.util.Map;
import java.util.Optional;
import java.util.Random;
import java.util.Set;

import de.fhdw.gaming.core.domain.DefaultGame;
import de.fhdw.gaming.core.domain.MoveChecker;
import de.fhdw.gaming.core.domain.ObserverFactoryProvider;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntField;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntGame;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntPlayer;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntState;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntStrategy;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.moves.VierGewinntMove;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.moves.factory.VierGewinntDefaultMoveFactory;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.moves.factory.VierGewinntMoveFactory;

/**
 * Implementation of a VierGewinntGame.
 * 
 * @author Robby Rabbitman
 *
 */
public class VierGewinntGameImpl
        extends DefaultGame<VierGewinntPlayer, VierGewinntState, VierGewinntMove, VierGewinntStrategy>
        implements VierGewinntGame {
    /**
     * The move factory.
     */
    private final VierGewinntMoveFactory moveFactory;

    /**
     * .
     * 
     * @param id
     * @param initialState
     * @param strategies
     * @param maxComputationTimePerMove
     * @param moveChecker
     * @param observerFactoryProvider
     * @throws IllegalArgumentException
     * @throws InterruptedException
     */
    public VierGewinntGameImpl(final int id, final VierGewinntState initialState,
            final Map<String, VierGewinntStrategy> strategies, final long maxComputationTimePerMove,
            final MoveChecker<VierGewinntPlayer, VierGewinntState, VierGewinntMove> moveChecker,
            final ObserverFactoryProvider observerFactoryProvider)
            throws IllegalArgumentException, InterruptedException {
        super(id, initialState, strategies, maxComputationTimePerMove, moveChecker, observerFactoryProvider);
        this.moveFactory = new VierGewinntDefaultMoveFactory();
    }

    @Override
    public Optional<VierGewinntMove> chooseRandomMove(final VierGewinntPlayer player,
            final VierGewinntState state) {
        final Set<VierGewinntField> fields = state.getBoard().getAllPlayableFields();
        final Optional<VierGewinntField> field =
                fields.stream().skip(new Random().nextInt(fields.size())).findFirst();
        if (field.isPresent()) {
            return Optional.of(this.moveFactory.createMove(field.get().getPosition()));
        }
        return Optional.empty();
    }

}
