package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.impl;

import java.util.Optional;

import de.fhdw.gaming.core.domain.PlayerState;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntFieldState;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntPlayer;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntState;

/**
 * The Implementation of a VierGewinntPlayer.
 * 
 * @author Robby Rabbitman
 *
 */
public class VierGewinntPlayerImpl implements VierGewinntPlayer {

    /**
     * The associated game state.
     */
    private final VierGewinntState gameState;

    /**
     * The name of this player.
     */
    private final String name;

    /**
     * true if yellow.
     * 
     */
    private final boolean usingYellow;

    /**
     * The Constructor.
     * 
     * @param gameState
     * @param name
     * @param usingYellow
     */
    public VierGewinntPlayerImpl(final VierGewinntState gameState, final String name, final boolean usingYellow) {
        super();
        this.gameState = gameState;
        this.name = name;
        this.usingYellow = usingYellow;

    }

    @Override
    public String getName() {
        return this.name;
    }

    @Override
    public PlayerState getState() {
        return this.gameState.getPlayerState(this.getName());
    }

    @Override
    public Optional<Double> getOutcome() {
        return this.gameState.getPlayerOutcome(this.getName());
    }

    @Override
    public boolean isUsingYellow() {
        return this.usingYellow;
    }

    @Override
    public VierGewinntFieldState getPlayerMark() {
        if (this.isUsingYellow()) {
            return VierGewinntFieldState.YELLOW;
        } else {
            return VierGewinntFieldState.RED;
        }
    }

    @Override
    public VierGewinntPlayer deepCopy(final VierGewinntState newGameState) {
        return new VierGewinntPlayerImpl(newGameState, this.name, this.isUsingYellow());
    }

}
