package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntBoard;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntField;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntPosition;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntRow;

/**
 * Implementation of a Row in a VierGewinntGame.
 *
 * @author Robby Rabbitman
 *
 */
public class ViergewinntRowImpl implements VierGewinntRow {

    /**
     * fields of this row.
     *
     */
    private final List<VierGewinntField> fields;

    /**
     * Create row with number of fields.
     *
     * @param numberOfFields
     * @param row
     * @param board
     * @throws GameException
     */
    public ViergewinntRowImpl(final Integer numberOfFields, final Integer row, final VierGewinntBoard board)
            throws GameException {
        super();
        this.fields = new ArrayList<>(numberOfFields);
        for (int i = 0; i < numberOfFields; i++) {
            this.fields.add(new VierGewinntFieldImpl(VierGewinntPosition.of(row, i), board));
        }
    }

    /**
     * copy.
     *
     * @param source
     * @param board  The board to associate with this row.
     */
    private ViergewinntRowImpl(final ViergewinntRowImpl source, final VierGewinntBoard board) {
        super();
        this.fields = source.getFields().stream().map((final VierGewinntField field) -> field.deepCopy(board))
                .collect(Collectors.toList());
    }

    @Override
    public List<VierGewinntField> getFields() {
        return this.fields;
    }

    @Override
    public VierGewinntRow deepCopy(final VierGewinntBoard target) {
        return new ViergewinntRowImpl(this, target);
    }

    @Override
    public String toString() {
        final StringBuilder out = new StringBuilder();
        for (final VierGewinntField field : this.fields.stream()
                .sorted((a, b) -> a.getPosition().getColumn() - b.getPosition().getColumn())
                .collect(Collectors.toList())) {
            out.append(field.getState().toString()).append(' ');
        }
        return out.toString();
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj == null || obj.getClass() != this.getClass()) {
            return false;
        } else if (obj == this) {
            return true;
        }

        final ViergewinntRowImpl otherRow = (ViergewinntRowImpl) obj;

        final Iterator<VierGewinntField> otherFields = otherRow.getFields().iterator();
        final Iterator<VierGewinntField> thisFields = this.getFields().iterator();

        while (otherFields.hasNext() && thisFields.hasNext()) {
            if (!(otherFields.next().equals(thisFields.next()))) {
                return false;
            }
        }

        return true;
    }

    /**
     * Returns Hashcode of this.
     */
    @Override
    public int hashCode() {
        return this.fields.hashCode();
    }
}
