package de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.moves.impl;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntPlayer;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntPosition;
import de.fhdw.gaming.ipspiel21.viergewinnt.core.domain.VierGewinntState;

/**
 * The Implementation of a VierGewinntMove.
 * 
 * @author Robby Rabbitman
 *
 */
public class VierGewinntMoveImpl extends AbstractMove {

    /**
     * Target position.
     * 
     */
    private final VierGewinntPosition position;

    /**
     * Move with target position.
     * 
     * @param position
     */
    public VierGewinntMoveImpl(final VierGewinntPosition position) {
        super();
        this.position = position;
    }

    @Override
    public void applyTo(final VierGewinntState state, final VierGewinntPlayer player) throws GameException {
        if (state.getBoard().getAllPlayableFields().stream().map(field -> field.getPosition())
                .filter(p -> p.equals(this.getPosition())).findAny().isEmpty()) {
            throw new GameException("Unvalid move");
        } else {
            state.getBoard().getFieldAt(this.getPosition()).changeState(player.getPlayerMark());
            state.moveCompleted(this);
        }
    }

    @Override
    public VierGewinntPosition getPosition() {
        return this.position;
    }

}
